@@ -1,6 +1,7 @@
 /* canon.c
  *
  * (c) 2006 Thierry MERLE <thierry.merle@free.fr>
+ * (c) 2009 Louis Lagendijk
  *
  * Code taken from epson.c. Commands taken from CanonUtil 0.07 
  * (http://xwtools.automatix.de/)
@@ -12,12 +13,7 @@
  * For details see file COPYING.
  */
 
-#include "inklevel.h"
-
-#ifdef DEBUG
-/* for isprint(), for debug prints */
-#include <ctype.h>
-#endif
+#include "config.h"
 
 #include <stdio.h>
 #include <sys/types.h>
@@ -26,14 +22,16 @@
 #include <string.h>
 #include <unistd.h>
 
-extern int my_atoi(char *t);
-extern int open_printer_device(const int port, const char *device_file, 
-                               const int portnumber);
-extern int read_from_printer(int fd, void *buf, size_t bufsize, 
-                             int nonblocking);
+#include "inklevel.h"
+#include "platform_specific.h"
+#include "util.h"
+#include "bjnp.h"
+#include "canon.h"
 
-int get_ink_level_canon(const int port, const char *device_file, 
-                        const int portnumber, struct ink_level *level);
+#ifdef DEBUG
+/* for isprint(), for debug prints */
+#include <ctype.h>
+#endif
 
 /* shortcut to compute string length */
 #define GET_STR_LENGTH(var) (sizeof((var))-1)
@@ -46,19 +44,28 @@ typedef unsigned short levelTab[MAX_CARTRIDGE_TYPES];
 
 /* decode "CHD" pattern. This gives the cartridge type that helps deducing the
    type and number of cartridges */
+
 static void decodeCHD(char *s, struct ink_level *level);
+
 /* decode "DOC" pattern. Operator Call tells which cartridge the operator must
    change. */
+
 static void decodeDOC(char *s, levelTab lt);
+
 /* decode "DWS" pattern. Warning State tells 
    which cartridge has low ink level */
+
 static void decodeDWS(char *s, levelTab lt);
+
 /* decode "CIR" pattern. Refined Ink levels, 
    tells the exact value of ink levels,
    but this function seems implemented on newer printers only */
-static void decodeCIR(char *s, levelTab lt);
+static void decodeCIR(char *s, struct ink_level *level);
 
-/* IMPORTANT NOTE: the number of cartridges is to be confirmed for each type */
+/* 
+ * IMPORTANT NOTE: the number of cartridges is to be confirmed for each type 
+ * of printer, EXCEPT for those printers that report the CIR-tag
+ */
 
 struct
 { char *model; /* The model identifier */
@@ -81,10 +88,18 @@ struct
      {{"BK,CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {"BK",1,{CARTRIDGE_BLACK}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"iP1600",
+     {{"BK,CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
+      {"BK",1,{CARTRIDGE_BLACK}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"S300",
      {{"VC",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"S500",
+     {{"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
+               CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"S520",
      {{"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
@@ -146,6 +161,9 @@ struct
     {"iP1500",
      {{"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"iP2000",
+     {{"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"iP4100",
      {{"VC",6,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,
@@ -172,7 +190,7 @@ struct
       {"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
       {NULL,RESPONSE_INVALID,{}}}},
-     {"iP3000",
+    {"iP3000",
      {{"VC",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
                CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
       {"CL",4,{CARTRIDGE_BLACK,CARTRIDGE_CYAN,
@@ -208,6 +226,12 @@ struct
     {"MP360",
      {{"CL",2,{CARTRIDGE_BLACK,CARTRIDGE_COLOR}},
       {NULL,RESPONSE_INVALID,{}}}},
+    {"MP530",
+     {{"VC",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
+      {"CL",5,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOBLACK,
+               CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW,}},
+      {NULL,RESPONSE_INVALID,{}}}},
     {"iP4000",
      {{"VC",5,{CARTRIDGE_PHOTOBLACK,CARTRIDGE_BLACK,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,
@@ -216,20 +240,28 @@ struct
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,
                CARTRIDGE_YELLOW}},
       {NULL,RESPONSE_INVALID,{}}}},
-   {"i9100",
+    {"i9100",
      {{"DS",6,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOCYAN,CARTRIDGE_PHOTOMAGENTA,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
       {"LS",6,{CARTRIDGE_BLACK,CARTRIDGE_PHOTOCYAN,CARTRIDGE_PHOTOMAGENTA,
                CARTRIDGE_CYAN,CARTRIDGE_MAGENTA,CARTRIDGE_YELLOW}},
-      {NULL,RESPONSE_INVALID,{}}}}
-};
+      {NULL,RESPONSE_INVALID,{}}}},
+    {"860i",
+     {{"VC",5,{CARTRIDGE_PHOTOBLACK,CARTRIDGE_BLACK,
+               CARTRIDGE_YELLOW,CARTRIDGE_MAGENTA,
+               CARTRIDGE_CYAN}},
+      {"CL",5,{CARTRIDGE_PHOTOBLACK,CARTRIDGE_BLACK,
+               CARTRIDGE_YELLOW,CARTRIDGE_MAGENTA,
+               CARTRIDGE_CYAN}},
+      {NULL,RESPONSE_INVALID,{}}}},
+  };
 
 const static int MAX_NUMBER_OF_MODELS = 
   sizeof(existingPrinters) / sizeof(existingPrinters[0]);
 
 /* Some Canon printers give a binary ink level indicator :
-  LOW (attributed to 20% remaining) or not LOW, 100% remaining.
-  Some other printers give more precise indicator, 40 or 70% remaining. */
+   LOW (attributed to 20% remaining) or not LOW, 100% remaining.
+   Some other printers give more precise indicator, 40 or 70% remaining. */
 #define LEVEL_LOW 20 /* 20% at which there is a warning */
 #define LEVEL_LOW_70 70
 #define LEVEL_LOW_40 40
@@ -261,62 +293,75 @@ int get_ink_level_canon(const int port, const char* device_file,
   int retry = 6; /* You can change this, but keep same parity */
   levelTab lt;
 
-  do {
-    fd = open_printer_device(port, device_file, portnumber);
-    if (fd < 0) {
-      return fd;
-    }
-
-    /* Get colors command */
-    length = makeCommand_canon(cmdGetColors,
-                               GET_STR_LENGTH(cmdGetColors),
-			       command);
-
-    i = write(fd, command, length);
-    if (i < length) {
-
-#ifdef DEBUG
-      printf("Could not send command to printer\n");
-#endif
-
-      close(fd);
-      return COULD_NOT_WRITE_TO_PRINTER;
-    }
-
-    length = read_from_printer(fd, buffer, BUFLEN, 0);
-    if (length <= 0) {
-
-#ifdef DEBUG    
-      printf("Could not read from printer\n");
-#endif
-      close(fd);
-      return COULD_NOT_READ_FROM_PRINTER;
-    }
-    /* Insert a terminator so that whe can do string operations */
-    buffer[length] = '\0';
-
-#ifdef DEBUG
-    printf("Command Response: \n");
-    for (i = 0; i<length; i++) {
-      if (isprint(buffer[i])) 
-        printf("%c", (unsigned char) buffer[i]);
-      else 
-        printf("\\x%02x", (unsigned char) buffer[i]);
-    }
-    printf("\n");
-#endif
+  if ((port == BJNP) || (port == CUSTOM_BJNP)) {
+    bjnp_get_printer_status(port, device_file,  portnumber, buffer);
 
     /* The command response is a list of semicolons-separated TOKEN:VALUE.
        example : DOC:4,00,NO;DWS:1512,1513;CHD:CL;
        First 2 bytes are response length. Ignoring them. */
+
     indexDOC = strstr(buffer+2, "DOC:");
     indexDWS = strstr(buffer+2, "DWS:");
     indexCHD = strstr(buffer+2, "CHD:");
     indexCIR = strstr(buffer+2, "CIR:");
-    
-    close(fd);
+  } else {
+    do {
+      fd = open_printer_device(port, device_file, portnumber);
+      if (fd < 0) {
+	return fd;
+      }
 
-  } while (!indexDOC && !indexDWS && !indexCHD && !indexCHD && --retry);
+      /* Get colors command */
+      length = makeCommand_canon(cmdGetColors,
+				 GET_STR_LENGTH(cmdGetColors),
+				 command);
+
+      i = write(fd, command, length);
+      if (i < length) {
+
+#ifdef DEBUG
+	printf("Could not send command to printer\n");
+#endif
+
+	close(fd);
+	return COULD_NOT_WRITE_TO_PRINTER;
+      }
+
+      length = read_from_printer(fd, buffer, BUFLEN, 0);
+      if (length <= 0) {
+
+#ifdef DEBUG    
+	printf("Could not read from printer\n");
+#endif
+	close(fd);
+	return COULD_NOT_READ_FROM_PRINTER;
+      }
+      /* Insert a terminator so that whe can do string operations */
+      buffer[length] = '\0';
+
+#ifdef DEBUG
+      printf("Command Response: \n");
+      for (i = 0; i<length; i++) {
+	if (isprint(buffer[i])) 
+	  printf("%c", (unsigned char) buffer[i]);
+	else 
+	  printf("\\x%02x", (unsigned char) buffer[i]);
+      }
+      printf("\n");
+#endif
+
+      /* The command response is a list of semicolons-separated TOKEN:VALUE.
+	 example : DOC:4,00,NO;DWS:1512,1513;CHD:CL;
+	 First 2 bytes are response length. Ignoring them. */
+      indexDOC = strstr(buffer+2, "DOC:");
+      indexDWS = strstr(buffer+2, "DWS:");
+      indexCHD = strstr(buffer+2, "CHD:");
+      indexCIR = strstr(buffer+2, "CIR:");
+    
+      close(fd);
+
+    } while (!indexDOC && !indexDWS && !indexCHD && !indexCHD && --retry);
+  }
 
   if (!indexDOC && !indexDWS && !indexCHD && !indexCHD) {
 	  
@@ -326,33 +371,35 @@ int get_ink_level_canon(const int port, const char* device_file,
 
     return COULD_NOT_PARSE_RESPONSE_FROM_PRINTER;
   }
-  /* decodeCHD -> Cartridge type <-
-     this fuction will update the level structure with default values depending
-     of the printer definition */
-  if(indexCHD) decodeCHD(indexCHD,level);
-  /* At this state, we must have a valid status.
-     otherwise that means the printer is not supported */
-  if(level->status == RESPONSE_INVALID)	{
-    return PRINTER_NOT_SUPPORTED;
-  }
-
-  /* before processing, set to 100% all ink levels. */
-  for(i=0;i<MAX_CARTRIDGE_TYPES;i++) {
-    lt[i] = 100;
-  }
-  /* Check DWS ->Warning State<- */
-  if(indexDWS) decodeDWS(indexDWS,lt);
-
-  /* Check DOC ->Operator Call<- */
-  if(indexDOC) decodeDOC(indexDOC,lt);
-
   /* Check CIR ->Ink Fill Detail<- exact ink level */
-  if(indexCIR) decodeCIR(indexCIR,lt);
+  if(indexCIR) 
+    decodeCIR(indexCIR,level);
+  else {
+    /* decodeCHD -> Cartridge type <-
+       this fuction will update the level structure with default values depending
+       of the printer definition */
+    if(indexCHD) decodeCHD(indexCHD,level);
+    /* At this state, we must have a valid status.
+       otherwise that means the printer is not supported */
+    if(level->status == RESPONSE_INVALID)	{
+      return PRINTER_NOT_SUPPORTED;
+    }
 
-/* Need to put the level tab computed by the decodexxx functions
-     into the struct inklevel before return */
-  for(i=0;i<MAX_CARTRIDGE_TYPES;i++) {
-    level->levels[i][INDEX_LEVEL] = lt[level->levels[i][INDEX_TYPE]];
+    /* before processing, set to 100% all ink levels. */
+    for(i=0;i<MAX_CARTRIDGE_TYPES;i++) {
+      lt[i] = 100;
+    }
+    /* Check DWS ->Warning State<- */
+    if(indexDWS) decodeDWS(indexDWS,lt);
+
+    /* Check DOC ->Operator Call<- */
+    if(indexDOC) decodeDOC(indexDOC,lt);
+
+    /* Need to put the level tab computed by the decodexxx functions
+       into the struct inklevel before return */
+    for(i=0;i<MAX_CARTRIDGE_TYPES;i++) {
+      level->levels[i][INDEX_LEVEL] = lt[level->levels[i][INDEX_TYPE]];
+    }
   }
 #ifdef DEBUG
   printf("Ink levels : \n");
@@ -509,8 +556,8 @@ static void decodeCHD(char *s, struct ink_level *level) {
         for(ic=0;ic<existingPrinters[mdl].chdTypes[i].numberOfColors;ic++) {
 	  level->levels[ic][INDEX_TYPE] =
 	    existingPrinters[mdl].chdTypes[i].cartridgeTypes[ic];
-	    /* by default ink level is 100. further indicators will give more
-	       information */
+	  /* by default ink level is 100. further indicators will give more
+	     information */
 	  level->levels[ic][INDEX_LEVEL] = 100;
 	}
 
@@ -526,32 +573,50 @@ static void decodeCHD(char *s, struct ink_level *level) {
 #endif
 }
 
-static void decodeCIR(char *s, levelTab lt) {
-  char *keys[9]={",K=",",BK=",",PBK=",",LC=",",LM=",",Y=",",M=",",C=",",CL="};
-  char *z;
-  int values[9]={0,0,0,0,0,0,0,0,0}, i;
+static void decodeCIR(char *s, struct ink_level *level) {
+  /*   char *keys[9]={",K=",",BK=",",PBK=",",LC=",",LM=",",Y=",",M=",",C=",",CL="}; */
+  /*  int values[9]={0,0,0,0,0,0,0,0,0}, i; */
 
+  struct {
+    short type;
+    char *key;
+  } cartridges[] = 
+      {
+	{CARTRIDGE_BLACK, ",K="},
+	{CARTRIDGE_BLACK, ",BK="},
+	{CARTRIDGE_PHOTOBLACK , ",PBK="}, 
+	{CARTRIDGE_PHOTOCYAN, ",LC="}, 
+	{CARTRIDGE_PHOTOMAGENTA, ",LM="}, 
+	{CARTRIDGE_YELLOW, ",Y="}, 
+	{CARTRIDGE_MAGENTA, ",M="}, 
+	{CARTRIDGE_CYAN, ",C="}, 
+	{CARTRIDGE_COLOR, ",CL="}
+      };
+#define NO_CARTRIDGES 9
+
+  char *cart_level;	/* inklevel reported by printer */
+  int i;		/* loop variable for cartridges table */
+  int level_index =0;	/* next avaiable position in level table */
+
+  /* skip CIR tag */
   s+=3;
+  
+  /* replace ":" by "," to ensure that all subtags start with "," */
   s[0]=',';
 
-  for(i=0;i<9;i++) { 
-    if( (z=strstr(s,keys[i])) != 0) {
-      sscanf(z+strlen(keys[i]),"%d",&values[i]);
+  for(i = 0; i < NO_CARTRIDGES; i++) { 
+    if ((cart_level=strstr(s,cartridges[i].key)) != 0) {
+      level->levels[level_index][INDEX_TYPE] = cartridges[i].type;
+      sscanf(cart_level+strlen(cartridges[i].key),
+             "%hd",&level->levels[level_index][INDEX_LEVEL]);
+      level_index++;
     }
   }
-
-  lt[CARTRIDGE_BLACK] = values[0]+values[1];
-  lt[CARTRIDGE_PHOTOBLACK] = values[2];
-  lt[CARTRIDGE_PHOTOCYAN] = values[3];
-  lt[CARTRIDGE_PHOTOMAGENTA] = values[4];
-  lt[CARTRIDGE_YELLOW] = values[5];
-  lt[CARTRIDGE_MAGENTA] = values[6];
-  lt[CARTRIDGE_CYAN] = values[7];
-  lt[CARTRIDGE_COLOR] = values[8];
+  level->status = RESPONSE_VALID;
 }
 
 static int makeCommand_canon(const char * /* in */ cmd, int cmdLen,
-		      char * /* out */ data) {
+			     char * /* out */ data) {
   int dataLen = 0;
   int length;
   char cmdHdr[]= "\x1b[K"; /* All commands begin with that, 
