@@ -36,6 +36,8 @@
  */
  
 
+#include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -60,7 +62,11 @@ int d4WrTimeout = WRTIMEOUT;
 int d4RdTimeout = RDTIMEOUT;
 int ppid        = 0;
 
+#ifdef DEBUG
+int debugD4     = 1;
+#else
 int debugD4     = 0;
+#endif
 
 typedef void (*signalHandler_t)(int);
 static signalHandler_t sig;
