@@ -8,15 +8,20 @@
  * For details see file COPYING.
  */
 
-#include "inklevel.h"
-#include "d4lib.h"
+#include "config.h"
 
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
+#include <signal.h>
 #include <sys/wait.h>
 #include <stdarg.h>
 
+#include "inklevel.h"
+#include "platform_specific.h"
+#include "epson_new.h"
+#include "d4lib.h"
+
 static int do_status_command_internal(void);
 static int initialize_printer();
 static void exit_packet_mode_old(int do_init);
@@ -37,9 +42,6 @@ extern int open_printer_device(const int port, const char* device_file,
                                const int portnumber);
 extern int read_from_printer(int fd, char *buf, int bufsize, int nonblocking);
 
-int get_ink_level_epson(const int port, const char *device_file, 
-                        const int portnumber, struct ink_level *level);
-
 static char printer_cmd[1025];
 static int bufpos = 0;
 static int isnew = 0;
@@ -642,13 +644,14 @@ static void do_new_status(char *buf, int bytes) {
   while (i < bytes) {
     unsigned hdr = buf[i];
     unsigned total_param_count = buf[i + 1];
+    unsigned param = buf[i + 2];
 
 #ifdef DEBUG
     printf("Header: %x param count: %d\n", hdr, total_param_count);
 #endif
 
     if (hdr == 0x0f) {	/* Always report ink */
-      size_t count = (total_param_count - 1) / 3;
+      size_t count = (total_param_count - 1) / param;
       ind = buf + i + 3;
 
 #ifdef DEBUG
@@ -685,7 +688,7 @@ static void do_new_status(char *buf, int bytes) {
 #endif
         }
 
-        ind += 3;
+        ind += param;
       }
     }
     i += total_param_count + 2;
