@@ -6,17 +6,13 @@
  * For details see file COPYING.
  */
 
+#include "config.h"
+
 #include <stdlib.h>
+
 #include "inklevel.h"
-
-extern int my_axtoi(char *);
-extern int my_atoi(char *);
-
-int parse_device_id_new_hp(char tags[NR_TAGS][BUFLEN], int n, 
-                           struct ink_level *level);
-int parse_device_id_old_hp(char [NR_TAGS][BUFLEN], int n, 
-                           struct ink_level *level);
-
+#include "util.h"
+#include "hp_new.h"
 
 /* This function parses the device id of a new HP printer
  * for example HP Deskjet 5550 
