@@ -1,6 +1,6 @@
 /* inklevel.h
  *
- * (c) 2003, 2004, 2005, 2006, 2007 Markus Heinz
+ * (c) 2003, 2004, 2005, 2006, 2007, 2009 Markus Heinz
  *
  * This software is licensed under the terms of the GPL.
  * For details see file COPYING.
@@ -9,8 +9,6 @@
 #ifndef INKLEVEL_H
 #define INKLEVEL_H
 
-//#define DEBUG 1
-
 #ifdef DEBUG
 #include <stdio.h> /* for the debug printfs */
 #endif
@@ -23,6 +21,8 @@
 #define USB 2
 #define CUSTOM_PARPORT 3
 #define CUSTOM_USB 4
+#define BJNP 5
+#define CUSTOM_BJNP 6
 
 /* Possible return values for get_ink_level() */
 
@@ -43,6 +43,8 @@
 #define COULD_NOT_PARSE_RESPONSE_FROM_PRINTER -14
 #define COULD_NOT_GET_CREDIT -15
 #define DEV_CUSTOM_USB_INACCESSIBLE -16
+#define BJNP_URI_INVALID -17
+#define BJNP_INVALID_HOSTNAME -18
 
 #define MODEL_NAME_LENGTH 100
 #define MAX_CARTRIDGE_TYPES 40
@@ -102,14 +104,4 @@ int get_ink_level(const int port, const char*device_file,
                   const int portnumber, struct ink_level *level);
 char *get_version_string(void);
 
-/* internal */
-
-#define BUFLEN 1024
-#define NR_TAGS 15
-
-#define OLD 1
-#define NEW 2
-
-#define LIBINKLEVEL_VERSION_STRING "libinklevel v0.7.3"
-
 #endif
